import pickle
import json
import uuid
from pathlib import Path
from bs4 import BeautifulSoup
from ..util import json_serial

# https://github.com/jmcarp/robobrowser/issues/96
# workaround issue with a monkeypatch
import werkzeug
werkzeug.cached_property = werkzeug.utils.cached_property
import robobrowser  # noqa:E402


def make_uuid_from_url(url):
    return uuid.uuid5(uuid.NAMESPACE_URL, url)


def load_json(filename):
    return json.load(open(filename, 'rb'))


def load_pickle(filename):
    return pickle.load(open(filename, 'rb'), encoding='utf8')


def dump_json(data, fileobj):
    json.dump(data, fileobj)


def dump_pickle(data, fileobj):
    pickle.dump(data, fileobj)


LOADERS = dict(json=load_json, pickle=load_pickle)
MODULES = dict(json=json, pickle=pickle)


class BaseCollector(object):
    def __init__(self):
        self.browser = robobrowser.RoboBrowser(parser='lxml')
        self.url = None
        self.response = None
        self.pageinfo = None
        self.content = ''
        self.soup = None

    def _make_soup(self, content):
        return BeautifulSoup(content, 'lxml')

    def retrieve_page(self, url=None):
        if url is None:
            if self.url is None:
                raise RuntimeError("No url set.")
            url = self.url
        else:
            self.url = url
        self.response = self.browser.open(url)
        # self.info = self.browser.response.headers
        self.info = dict()
        for key in self.browser.response.headers:
            self.info[key.lower()] = self.browser.response.headers[key]
        self.content = self.browser.response.content
        self.soup = self.browser.parsed

    def set_url(self, url):
        self.url = url
        self.response = None
        self.pageinfo = None
        self.content = ''
        self.soup = None


class CacheCollector(BaseCollector):
    def __init__(self, cachedir='cache', format='pickle'):
        super(CacheCollector, self).__init__()
        self.cachedir = Path(cachedir)
        if not self.cachedir.exists():
            self.cachedir.mkdir(parents=True)
        self.format = format

    def filename(self, url):
        filename = '{}.{}'.format(make_uuid_from_url(url), self.format)
        return self.cachedir / filename

    def _load_json(self, filename):
        return json.load(filename.open('r'))

    def _load_pickle(self, filename):
        return pickle.load(filename.open('rb'))

    def get_from_cache(self, url):
        filename = self.filename(url)
        if filename.exists():
            if self.format == 'json':
                return self._load_json(filename)
            elif self.format == 'pickle':
                return self._load_pickle(filename)
            else:
                raise RuntimeError('bad format: {}'.format(self.format))
        else:
            return None

    def _dump_json(self, data, filename):
        with filename.open('w') as outfile:
            json.dump(data, outfile, default=json_serial)

    def _dump_pickle(self, data, filename):
        with filename.open('wb') as outfile:
            pickle.dump(data, outfile)

    def save_to_cache(self, url):
        self.retrieve_page(url)
        data = dict(info=self.info, content=self.content, url=self.url)
        filename = self.filename(url)
        if self.format == 'json':
            self._dump_json(data, filename)
        elif self.format == 'pickle':
            self._dump_pickle(data, filename)
        else:
            raise RuntimeError("unknown format {}".format(self.format))



import os
import xml.etree.ElementTree as ET
import csv

import requests

scummvm_compat_xml_url = 'https://github.com/scummvm/scummvm-web/raw/master/data/compatibility/compat-DEV.xml'  # noqa


def get_scummvm_compat_xml_remote(url):
    response = requests.get(url)
    if not response.ok:
        raise RuntimeError("request returned %s" % response)
    return response.content


def get_scummvm_compat_xml(url):
    filename = 'compat-DEV.xml'
    if not os.path.isfile(filename):
        content = get_scummvm_compat_xml_remote(url)
        with open(filename, 'w') as outfile:
            outfile.write(content)
    else:
        print("%s exists." % filename)


def get_one_element(element, tagname):
    elements = element.findall(tagname)
    if len(elements) != 1:
        msg = "Unable to get one element %s with tag %s"
        raise RuntimeError(msg % (element, tagname))
    return elements[0]


def parse_game_element(element):
    data = dict()
    for key in ['name', 'target', 'support_level', 'notes']:
        data[key] = get_one_element(element, key).text.strip()
    return data


def parse_company_tag(element, gdict):
    name_el = get_one_element(element, 'name')
    company = name_el.text
    if company in gdict:
        raise RuntimeError("Company %s already in dictionary" % company)
    gdict[company] = dict()
    games_el = get_one_element(element, 'games')
    for el in games_el:
        data = parse_game_element(el)
        target = data['target']
        if target in gdict[company]:
            raise RuntimeError("Duplicate target %s" % target)
        gdict[company][data['target']] = data


def make_target_data(filename):
    games = dict()
    tree = ET.parse(filename)
    root = tree.getroot()
    for child in root:
        if child.tag == 'company':
            parse_company_tag(child, games)
    targets = dict()
    for company in games:
        for target in games[company]:
            if target in targets:
                raise RuntimeError("Already processed target %s" % target)
            targets[target] = games[company][target]
            targets[target]['company'] = company
    return targets


def make_target_csv(targets):
    filename = 'targets.csv'
    if os.path.isfile(filename):
        return
    keys = list(targets.keys())
    print("Targets", len(keys))
    keys.sort()
    fields = ['target', 'name', 'company', 'support_level', 'notes']
    with open(filename, 'w') as outfile:
        writer = csv.DictWriter(outfile,
                                fieldnames=fields, quoting=csv.QUOTE_ALL)
        writer.writeheader()
        for key in keys:
            writer.writerow(targets[key])
            print(targets[key])



import io
import lzma
import json
from uuid import UUID

import warnings
from datetime import datetime, date

import sqlalchemy as sa
from sqlalchemy import PickleType

from sqlalchemy import engine_from_config
from sqlalchemy.orm import sessionmaker

# here is the common base from pyramid alchemy template
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.schema import MetaData
from sqlalchemy_utils import Timestamp

# http://stackoverflow.com/questions/4617291/how-do-i-get-a-raw-compiled-sql-query-from-a-sqlalchemy-expression
from sqlalchemy.sql import compiler
from psycopg2.extensions import adapt as sqlescape
import transaction


def compile_query(query):
    dialect = query.session.bind.dialect
    statement = query.statement
    comp = compiler.SQLCompiler(dialect, statement)
    comp.compile()
    enc = dialect.encoding
    params = {}
    for k, v in comp.params.items():
        if isinstance(v, str):
            v = v.encode(enc)
        params[k] = sqlescape(v)
    return (comp.string.encode(enc) % params).decode(enc)


# Recommended naming convention used by Alembic, as various different database
# providers will autogenerate vastly different names making migrations more
# difficult. See: http://alembic.zzzcomputing.com/en/latest/naming.html
NAMING_CONVENTION = {
    "ix": 'ix_%(column_0_label)s',
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s"
}

metadata = MetaData(naming_convention=NAMING_CONVENTION)
Base = declarative_base(metadata=metadata)


# inspired by ziggurat_foundations
class BaseModel(object):
    @classmethod
    def _get_keys(cls):
        'returns column names for this model'
        return sa.orm.class_mapper(cls).c.keys()

    @classmethod
    def get_primary_key(cls):
        'returns primary key'
        return sa.orm.class_mapper(cls).primary_key

    def to_dict(self):
        data = {}
        for key in self._get_keys():
            data[key] = getattr(self, key)
        return data

    def get_table_column_names(self):
        return self._get_keys()


class SerialBase(BaseModel):
    def serialize(self):
        data = dict()
        table = self.__table__
        for column in table.columns:
            name = column.name
            try:
                pytype = column.type.python_type
            except NotImplementedError:
                if type(column.type) is PickleType:
                    value = getattr(self, name)
                    if type(value) not in [dict, list]:
                        # ignore column
                        continue
                else:
                    msg = "{}({}) Not Implemented.".format(column, column.type)
                    warnings.warn(msg)
                    continue
            value = getattr(self, name)
            if pytype is datetime or pytype is date:
                if value is not None:
                    value = value.isoformat()
            elif pytype is UUID:
                if value is not None:
                    value = str(value)
            data[name] = value
        return data


class TimeStampMixin(SerialBase, Timestamp):
    @property
    def created_at(self):
        warnings.warn("created_at no longer needed")
        return super(Timestamp, self).created

    @property
    def updated_at(self):
        warnings.warn("updated_at no longer needed")
        return super(Timestamp, self).updated


def _make_db_session(dburl, create_all=False, baseclass=None):
    settings = {'sqlalchemy.url': dburl}
    engine = engine_from_config(settings)
    if create_all:
        baseclass.metadata.create_all(engine)
    session_class = sessionmaker()
    session_class.configure(bind=engine)
    return session_class


def make_sqlite_session(filename, create_all=False, baseclass=None):
    dburl = "sqlite:///%s" % filename
    return _make_db_session(dburl, create_all=create_all,
                            baseclass=baseclass)


def make_postgresql_session(dburl, create_all=False, baseclass=None):
    return _make_db_session(dburl, create_all=create_all,
                            baseclass=baseclass)


# This can use a lot of memory.  This is only useful for
# smaller datasets.
def export_models(session, models):
    data = dict()
    output = io.BytesIO()
    with transaction.manager:
        with lzma.LZMAFile(output, 'w') as zfile:
            for model in models:
                q = session.query(model)
                name = model.__name__
                print("Dumping {}".format(name))
                mlist = [m.serialize() for m in q]
                data[name] = mlist
                print("Exported {}".format(name))
            content = json.dumps(data).encode()
            zfile.write(content)
    del data
    return output.getvalue()



import unittest


class ChunksTest(unittest.TestCase):
    def test_chunks1(self):
        from ..base import chunks
        result = list(chunks(b'abcdefghi', 4))
        expected = [b'abcd', b'efgh', b'i']
        self.assertEqual(result, expected)

    def test_chunks2(self):
        from ..base import chunks
        result = list(chunks(b'abcdefghi', 3))
        expected = [b'abc', b'def', b'ghi']
        self.assertEqual(result, expected)



# column types
from .githubdb import GitHubUser, GitHubRepo


class NoNamedUserError(Exception):
    pass


user_attributes = [
    'id',
    'login',
    'avatar_url',
    'gravatar_id',
    'url',
    'html_url',
    'followers_url',
    'following_url',
    'gists_url',
    'starred_url',
    'subscriptions_url',
    'organizations_url',
    'repos_url',
    'events_url',
    'received_events_url',
    'type',
    'name',
    'company',
    'blog',
    'location',
    'email',
    'hireable',
    'bio',
    'public_repos',
    'public_gists',
    'followers',
    'following',
]


repo_attributes = [
    'id',
    'name',
    'full_name',
    'description',
    'private',
    'fork',
    'default_branch',
    'homepage',
    'url',
    'html_url',
    'has_issues',
    'has_wiki',
    'has_downloads',
    'size',
    'forks_count',
    'stargazers_count',
    'open_issues_count',
    'pushed_at',
]


def _make_dbobject(robj, dbclass, attributes):
    dbobj = dbclass()
    for attr in attributes:
        setattr(dbobj, attr, getattr(robj, attr))
    for attr in ['created_at', 'updated_at']:
        dbattr = '%s_gh' % attr
        setattr(dbobj, dbattr, getattr(robj, attr))
    dbobj.pickle = robj
    return dbobj


def make_dbuser(user):
    return _make_dbobject(user, GitHubUser, user_attributes)


def make_dbrepo(repo):
    dbrepo = _make_dbobject(repo, GitHubRepo, repo_attributes)
    dbrepo.owner_id = repo.owner.id
    return dbrepo


def add_dbuser(session, dbuser):
    session.add(dbuser)
    return session.merge(dbuser)

# if repo_owner is not None, it is repo.owner from github


def add_dbrepo(session, dbrepo, repo_owner=None, verbose=False):
    owner = session.query(GitHubUser).get(dbrepo.owner_id)
    if owner is None:
        if repo_owner is None:
            msg = "No user in database owning repo %s" % dbrepo.full_name
            raise NoNamedUserError(msg)
        if verbose:
            print("Add new user", repo_owner.login)
        dbuser = make_dbuser(repo_owner)
        dbuser = add_dbuser(session, dbuser)
    session.add(dbrepo)
    return session.merge(dbrepo)


def _import_repos(session, repolist, verbose=False):
    for repo in repolist:
        dbrepo = session.query(GitHubRepo).get(repo.id)
        if dbrepo is None:
            if verbose:
                print("Add new repo", repo.full_name)
            dbrepo = make_dbrepo(repo)
            dbrepo = add_dbrepo(session, dbrepo, repo_owner=repo.owner,
                                verbose=verbose)


def import_basic_data(session, ghub, commit=False, verbose=False):
    user = ghub.get_user()
    user_id = user.id
    dbuser = session.query(GitHubUser).get(user_id)
    if dbuser is None:
        if verbose:
            print("Add main user", user.login)
        dbuser = make_dbuser(user)
        session.add(dbuser)
    # import main repos
    _import_repos(session, user.get_repos(), verbose=verbose)
    # import starred repos
    _import_repos(session, user.get_starred(), verbose=verbose)
    if commit:
        if verbose:
            print("Committing Data...")
        session.commit()



from sqlalchemy import Column, ForeignKey

# column types
from sqlalchemy import Integer, Unicode
from sqlalchemy import UnicodeText
from sqlalchemy import Boolean
from sqlalchemy import PickleType
from sqlalchemy import DateTime
from sqlalchemy import BigInteger


from hornstone.alchemy import TimeStampMixin


####################################
#  Data Types                     ##
####################################

# GitAnnexBackendType = Enum('SHA256', 'SHA256E',
#                           name='gitannex_backend_type_enum')


####################################
#  Tables                         ##
####################################
#


vccol = Unicode(200)


class GitHubUser(TimeStampMixin):
    __tablename__ = 'ghub_users'
    id = Column(BigInteger, primary_key=True)
    login = Column(vccol, unique=True)
    avatar_url = Column(vccol)
    gravatar_id = Column(vccol)
    url = Column(vccol)
    html_url = Column(vccol)
    followers_url = Column(vccol)
    following_url = Column(vccol)
    gists_url = Column(vccol)
    starred_url = Column(vccol)
    subscriptions_url = Column(vccol)
    organizations_url = Column(vccol)
    repos_url = Column(vccol)
    events_url = Column(vccol)
    received_events_url = Column(vccol)
    type = Column(vccol)
    name = Column(vccol)
    company = Column(vccol)
    blog = Column(vccol)
    location = Column(vccol)
    email = Column(vccol)
    hireable = Column(Boolean)
    bio = Column(vccol)
    public_repos = Column(Integer)
    public_gists = Column(Integer)
    followers = Column(Integer)
    following = Column(Integer)
    created_at_gh = Column(DateTime)
    updated_at_gh = Column(DateTime)
    pickle = Column(PickleType)


class GitHubRepo(TimeStampMixin):
    __tablename__ = 'ghub_repos'
    id = Column(BigInteger, primary_key=True)
    owner_id = Column(BigInteger, ForeignKey('ghub_users.id'))
    name = Column(vccol)
    full_name = Column(vccol)
    description = Column(UnicodeText)
    private = Column(Boolean)
    fork = Column(Boolean)
    default_branch = Column(vccol)
    homepage = Column(vccol)
    url = Column(vccol)
    html_url = Column(vccol)
    has_issues = Column(Boolean)
    has_wiki = Column(Boolean)
    has_downloads = Column(Boolean)
    size = Column(Integer)
    forks_count = Column(Integer)
    stargazers_count = Column(Integer)
    open_issues_count = Column(Integer)
    pushed_at = Column(DateTime)
    created_at_gh = Column(DateTime)
    updated_at_gh = Column(DateTime)
    pickle = Column(PickleType)


####################################
#  Relationships                  ##
####################################

# AnnexFile.key = relationship(AnnexKey, backref='files')
# ArchiveFile.file = relationship(AnnexFile)
# ArchiveFile.entries = relationship(ArchiveEntry, backref='archive')
# ArchiveEntry.key = relationship(ArchiveEntryKey, backref='entries')
# RepoFile.file = relationship(AnnexFile, backref='repositories')



import os
from configparser import ConfigParser, NoSectionError

main_config = ConfigParser()
main_config.read('config/main.ini')

scummvm_config = ConfigParser()
scummvm_config.read('config/scummvm.ini')

add_sdl_var = False
if 'main' in scummvm_config.sections():
    cfgopt = 'add_sdl_environ_var'
    add_sdl_var = scummvm_config['main'].getboolean(cfgopt, False)
if add_sdl_var:
    os.environ['SDL_MOUSE_RELATIVE'] = '0'


def build_cmd(directory):
    mdata = scummvm_config.items('main')
    cmd = [mdata['scummvm_bin'], '-e', mdata['music_driver']]
    try:
        if scummvm_config.getboolean('main', 'full_screen'):
            cmd.append('-f')
    except NoSectionError:
        pass
    cmd += ['-g', mdata['graphics_scaler']]
    cmd += ['-p', directory]
    return cmd



from datetime import datetime, date
from configparser import ConfigParser


# https://stackoverflow.com/a/22238613
def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    if isinstance(obj, bytes):
        return obj.decode()
    raise TypeError("Type %s not serializable" % type(obj))


def getboolean(value):
    if value.lower() not in ConfigParser.BOOLEAN_STATES:
        raise ValueError('Not a boolean: {}'.format(value))
    return ConfigParser.BOOLEAN_STATES[value.lower()]



import unittest


class GetbooleanTest(unittest.TestCase):
    def _test_methods(self, value, expected):
        from ..util import getboolean
        methods = ['lower', 'title', 'upper']
        for method in methods:
            test_value = getattr(value, method)()
            params = dict(test_value=test_value,
                          method=method,
                          value=value,
                          expected=expected)
            msg = "value is {}, expected result {}".format(value, expected)
            with self.subTest(msg, **params):
                result = getboolean(test_value)
                self.assertEqual(result, expected)

    def test_truthness(self):
        values = ['true', 'yes', '1', 'on']
        for value in values:
            self._test_methods(value, True)

    def test_falseness(self):
        values = ['false', 'no', '0', 'off']
        for value in values:
            self._test_methods(value, False)

    def test_bad_values(self):
        from ..util import getboolean
        values = ['f', 't', 'y', 'n', '2']
        for value in values:
            msg = "value is {}".format(value)
            with self.subTest(msg=msg, value=value):
                self.assertRaises(ValueError, getboolean, value)

    def test_bad_types(self):
        from ..util import getboolean
        values = [0, 1, 2, 0.0, True, False]
        for value in values:
            msg = "value is {}".format(value)
            with self.subTest(msg=msg, value=value):
                self.assertRaises(AttributeError, getboolean, value)



from configparser import ConfigParser
from io import StringIO

from sqlalchemy import Column
from sqlalchemy import Boolean
from sqlalchemy import Unicode, UnicodeText
from sqlalchemy import ForeignKey, PickleType
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship
from sqlalchemy_utils import UUIDType

from ..alchemy import TimeStampMixin
from .base import BaseUUIDMixin
from .base import BaseShortNameUUIDMixin


class UserMixin(BaseUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'users'

    @declared_attr
    def username(self):
        return Column(Unicode(50), unique=True)

    @declared_attr
    def fullname(self):
        return Column(Unicode(150), unique=True)

    @declared_attr
    def email(self):
        return Column(Unicode(150), unique=True)

    @declared_attr
    def active(self):
        return Column(Boolean(name='user_active'), default=True)

    @declared_attr
    def password(self):
        return Column(Unicode(150))

    @declared_attr
    def settings(self):
        return Column(PickleType)

    @property
    def user_name(self):
        return super(TimeStampMixin, self).username

    def get_groups(self):
        return [g.name for g in self.groups]


class UserConfigMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'user_config'

    @declared_attr
    def id(self):
        return Column(UUIDType, ForeignKey('users.id'), primary_key=True)

    @declared_attr
    def text(self):
        return Column(UnicodeText)

    def get_config(self):
        c = ConfigParser()
        c.readfp(StringIO(self.text))
        return c

    def set_config(self, config):
        cfile = StringIO()
        config.write(cfile)
        cfile.seek(0)
        text = cfile.read()
        self.text = text


class GroupMixin(BaseShortNameUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'groups'

    @declared_attr
    def description(self):
        return Column(UnicodeText)

    @declared_attr
    def users(self):
        return relationship('User', secondary='group_user',
                            backref='groups')


class UserGroupMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'group_user'

    @declared_attr
    def group_id(self):
        return Column(UUIDType,
                      ForeignKey('groups.id',
                                 onupdate='CASCADE',
                                 ondelete='CASCADE'),
                      primary_key=True)

    @declared_attr
    def user_id(self):
        return Column(UUIDType,
                      ForeignKey('users.id',
                                 onupdate='CASCADE',
                                 ondelete='CASCADE'),
                      primary_key=True)


# User.config = relationship(UserConfig, uselist=False, lazy='subquery')



from github import Github


def make_client(username, password):
    return Github(username, password)


def make_portal(creds):
    mcreds = dict(creds.items('main'))
    return make_client(mcreds['user'], mcreds['password'])



#!/usr/bin/env python
import os
from datetime import datetime
import zipfile
import tempfile
import subprocess

from unipath.path import Path as path
# from unipath import FILES, DIRS, LINKS

import rarfile

from .base import get_sha256sum


rarfile.UNRAR_TOOL = 'rar'
rarfile.PATH_SEP = os.path.sep

# date_time and CRC handled separately
COMPATIBLE_ARCHIVE_INFO_ATTRIBUTES = ['comment', 'compress_size',
                                      'filename',
                                      'file_size', 'volume']
COMMON_ARCHIVE_INFO_ATTRIBUTES = ['compress_type', 'extract_version',
                                  'header_offset', 'orig_filename']
ONLY_ZIP_INFO_ATTRIBUTES = ['create_system', 'create_version',
                            'external_attr', 'extra', 'flag_bits',
                            'internal_attr', 'reserved']
ONLY_RAR_INFO_ATTRIBUTES = ['add_size', 'arctime', 'atime', 'ctime',
                            'file_offset', 'flags', 'header_base',
                            'header_crc', 'header_data', 'header_size',
                            'host_os', 'mode', 'mtime',
                            'name_size', 'salt', 'type',
                            'volume_file']

_COMMON = COMPATIBLE_ARCHIVE_INFO_ATTRIBUTES + COMMON_ARCHIVE_INFO_ATTRIBUTES
ZIP_INFO_ATTRIBUTES = _COMMON + ONLY_ZIP_INFO_ATTRIBUTES
RAR_INFO_ATTRIBUTES = _COMMON + ONLY_RAR_INFO_ATTRIBUTES
ARCHIVE_ATTRIBUTES = dict(zip=ZIP_INFO_ATTRIBUTES, rar=RAR_INFO_ATTRIBUTES)
ARCHIVE_FILECLASS = dict(zip=zipfile.ZipFile, rar=rarfile.RarFile)


def parse_archive_info(info, archive_type):
    data = dict()
    for att in ARCHIVE_ATTRIBUTES[archive_type]:
        value = getattr(info, att)
        if type(value) is str:
            value = str(value, errors='replace')
        data[att] = value
    # handle CRC for file
    data['crc'] = info.CRC
    data['date_time'] = datetime(*info.date_time)
    return data


def get_archive_type(filename):
    lname = filename.lower()
    if lname.endswith('.zip'):
        archive_type = 'zip'
    elif lname.endswith('.rar'):
        archive_type = 'rar'
    else:
        raise RuntimeError("unable to handle archive %s" % filename)
    return archive_type


def get_extracted_info(parent_dir, fileinfo, sha256sum=False):
    parent_dir = path(parent_dir)
    filename = fileinfo.filename
    try:
        extracted_name = path(parent_dir, filename)
    except UnicodeEncodeError:
        print("UnicodeEncodeError with %s" % filename)
        filename = filename.encode('utf8')
        extracted_name = path(parent_dir, filename)
        # make sure extracted_name exists
        if not extracted_name.isfile():
            raise RuntimeError("%s is not a file" % extracted_name)
    ifile = open(extracted_name)
    ifile.seek(0, 2)
    file_size = ifile.tell()
    ifile.seek(0)
    cksum = None
    if sha256sum:
        cksum = get_sha256sum(ifile)
    return file_size, cksum


def parse_rar_archive(filename, sha256sum=False):
    print("Creating infolist for %s" % filename)
    with rarfile.RarFile(filename, 'r') as afile:
        infolist = afile.infolist()
    print("Infolist created for %s" % filename)
    tmpdir = tempfile.mkdtemp(prefix='extracted-rar-')
    cmd = ['rar', 'x', filename, tmpdir]
    subprocess.check_call(cmd)
    here = path.cwd()
    os.chdir(tmpdir)
    td = path(tmpdir)
    print("Parsing info")
    entries = list()
    for ainfo in infolist:
        if ainfo.isdir():
            print("Skipping directory %s" % ainfo.filename)
            continue
        parsed = parse_archive_info(ainfo, 'rar')
        parsed['bytesize'] = ainfo.file_size
        size, cksum = get_extracted_info(td, ainfo, sha256sum=sha256sum)
        if size != ainfo.file_size:
            raise RuntimeError("Sizes don't match %s" % ainfo.filename)
        parsed['sha256sum'] = cksum
        entries.append(parsed)
    cmd = ['rm', '-fr', tmpdir]
    subprocess.check_call(cmd)
    os.chdir(here)
    return entries


def parse_zip_archive(filename, sha256sum=False):
    entries = list()
    with zipfile.ZipFile(filename, 'r') as afile:
        for ainfo in afile.infolist():
            parsed = parse_archive_info(ainfo, 'zip')
            parsed['bytesize'] = ainfo.file_size
            if sha256sum:
                fileobj = afile.open(ainfo)
                sha256sum = get_sha256sum(fileobj)
                parsed['sha256sum'] = sha256sum
            entries.append(parsed)
    return entries


archive_parser = dict(zip=parse_zip_archive, rar=parse_rar_archive)


def parse_archive_file(filename, sha256sum=False):
    archive_type = get_archive_type(filename)
    try:
        return archive_parser[archive_type](filename, sha256sum=sha256sum)
    except zipfile.BadZipfile:
        print("WARNING, bad zip file: %s" % filename)
        return list()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'more-itertools',
    'SQLAlchemy',
    'psycopg2',        # dbapi for postgresql
    'transaction',
    'sqlalchemy-utils',
    'robobrowser',
    'requests',
    'beautifulsoup4',
    'lxml',
    'rarfile',
    'unipath',
    'PyGithub',
    'gitpython',
    'passlib',
]
setup_requirements = [
    'pytest-runner',
    # TODO(umeboshi2): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'pytest',
    # TODO: put package test requirements here
]


setup(
    name='hornstone',
    version='0.1.23',
    description="A bunch of rocks",
    long_description=readme,
    author="Joseph Rawson",
    author_email='joseph.rawson.works@gmail.com',
    url='https://github.com/umeboshi2/hornstone',
    license='Public Domain',
    packages=find_packages(include=['hornstone', 'hornstone.*']),
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='hornstone',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import relationship
from sqlalchemy_utils import UUIDType

from ..alchemy import TimeStampMixin
from .base import BaseUUIDMixin, BaseNameUUIDMixin
# from .base import UserOwnedMixin


class PersonMixin(BaseNameUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'people'

    @declared_attr
    def blogs(self):
        return relationship('Blog', backref='owner')

    @declared_attr
    def posts(self):
        return relationship('Post', backref='author')

    @declared_attr
    def comments(self):
        return relationship('Comment', backref='author')


class BlogMixin(BaseUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'blogs'

    @declared_attr
    def title(self):
        return sa.Column(sa.UnicodeText)

    @declared_attr
    def owner_id(self):
        return sa.Column(UUIDType, sa.ForeignKey('people.id'))

    @declared_attr
    def posts(self):
        return relationship('Post', backref='blog')


class PostMixin(BaseUUIDMixin):
    @declared_attr
    def __tablename__(self):
        return 'posts'

    @declared_attr
    def title(self):
        return sa.Column(sa.UnicodeText)

    @declared_attr
    def content(self):
        return sa.Column(sa.UnicodeText)

    @declared_attr
    def published_at(self):
        return sa.Column(sa.DateTime,
                         nullable=False, default=sa.func.now())

    @declared_attr
    def blog_id(self):
        return sa.Column(UUIDType, sa.ForeignKey('blogs.id'))

    @declared_attr
    def author_id(self):
        return sa.Column(UUIDType,
                         sa.ForeignKey('people.id'), nullable=False)

    @declared_attr
    def comments(self):
        return relationship('Comment', backref='post')


class CommentMixin(TimeStampMixin):
    @declared_attr
    def __tablename__(self):
        return 'comments'

    @declared_attr
    def comments_id(self):
        return sa.Column(UUIDType, primary_key=True)

    @declared_attr
    def content(self):
        return sa.Column(sa.UnicodeText)

    @declared_attr
    def author_id(self):
        return sa.Column(UUIDType,
                         sa.ForeignKey('people.id'), nullable=False)

    @declared_attr
    def post_id(self):
        return sa.Column(UUIDType, sa.ForeignKey('posts.id'))

    @declared_attr
    def type(self):
        return sa.Column(sa.UnicodeText)
